package com.lucasjosino.on_audio_room.databases

import android.content.Context
import androidx.room.Database
import androidx.room.Room
import androidx.room.RoomDatabase
import com.lucasjosino.on_audio_room.OnAudioRoomPlugin
import com.lucasjosino.on_audio_room.databases.favorites.OnFavoritesDao
import com.lucasjosino.on_audio_room.databases.favorites.OnFavoritesEntity
import com.lucasjosino.on_audio_room.databases.lastPlayed.OnLastPlayedDao
import com.lucasjosino.on_audio_room.databases.lastPlayed.OnLastPlayedEntity
import com.lucasjosino.on_audio_room.databases.mostPlayed.OnMostPlayedDao
import com.lucasjosino.on_audio_room.databases.mostPlayed.OnMostPlayedEntity
import com.lucasjosino.on_audio_room.databases.playlists.OnPlaylistDao
import com.lucasjosino.on_audio_room.databases.playlists.OnPlaylistEntity
import com.lucasjosino.on_audio_room.databases.playlists.playlistSongs.OnPlaylistSongsDao
import com.lucasjosino.on_audio_room.databases.playlists.playlistSongs.OnPlaylistSongsEntity

@Database(entities = [
    OnFavoritesEntity::class,
    OnMostPlayedEntity::class,
    OnLastPlayedEntity::class,
    OnPlaylistEntity::class,
    OnPlaylistSongsEntity::class,
    OnSongEntity::class], version = 1, exportSchema = false)
abstract class OnAudioDatabase : RoomDatabase() {
    abstract fun onFavoritesDao() : OnFavoritesDao
    abstract fun onMostPlayedDao() : OnMostPlayedDao
    abstract fun onLastPlayedDao() : OnLastPlayedDao
    abstract fun onPlaylistDao() : OnPlaylistDao
    abstract fun onPlaylistSongDao(): OnPlaylistSongsDao

    companion object {
        @Volatile private var instance: OnAudioDatabase? = null

        fun initDatabase(context: Context) : OnAudioDatabase {
            val tempInstance = instance
            if (tempInstance != null) {
                return tempInstance
            }
            synchronized(this) {
                val instance = Room.databaseBuilder(
                    context,
                    OnAudioDatabase::class.java,
                    OnAudioRoomPlugin().getDbName(context)!!
                )
                    // All migrations
                    .addMigrations()
                    .build()
                this.instance = instance
                return instance
            }
        }

        fun closeDatabase(context: Context) : Boolean {
            val tempDatabase = initDatabase(context)
            return if (tempDatabase.isOpen) {
                tempDatabase.close()
                instance = null
                true
            } else false
        }
    }
}
