package com.lucasjosino.on_audio_room.utils

import com.lucasjosino.on_audio_room.databases.OnSongEntity
import com.lucasjosino.on_audio_room.databases.favorites.OnFavoritesEntity
import com.lucasjosino.on_audio_room.databases.lastPlayed.OnLastPlayedEntity
import com.lucasjosino.on_audio_room.databases.mostPlayed.OnMostPlayedEntity
import com.lucasjosino.on_audio_room.databases.playlists.OnPlaylistEntity
import com.lucasjosino.on_audio_room.databases.playlists.playlistSongs.OnPlaylistSongsEntity
import java.util.*
import kotlin.collections.ArrayList
import kotlin.collections.HashMap

//Default id for any of the entities
//The value isn't important because this id will be auto increased/generated.
private const val defaultId = 0

//This part it's a Kotlin extension, here we will convert Map/List from Dart and convert in Entity.

//Basic song/audio information
fun MutableMap<String, Any>.toAudioEntity(): OnSongEntity {
    return OnSongEntity(
        lastData = this["last_data"].toString(),
        displayName = this["display_name"].toString(),
        id = this["id"].toString().toLong(),
        album = this["album"].toString(),
        albumId = this["album_id"].toString().toLong(),
        artist = this["artist"].toString(),
        artistId = this["artist_id"].toString().toLong(),
        dateAdded = this["date_added"].toString().toLong(),
        duration = this["duration"].toString().toLong(),
        title = this["title"].toString(),
        artwork = this["artwork"].toString()
    )
}

//Favorites information with song/audio information
fun MutableMap<String, Any>.toFavoritesEntity(): OnFavoritesEntity {
    return OnFavoritesEntity(
        dbFavId = defaultId,
        this.toAudioEntity()
    )
}

//LastPlayed information with song/audio information
fun MutableMap<String, Any>.toLastPlayedEntity(autoTimer: Boolean = false): OnLastPlayedEntity {
    return OnLastPlayedEntity(
        dbLPId = defaultId,
        this.toAudioEntity(),
        timePlayed = this["time_played"].toString().toLong(),
        lastTimePlayed = if (!autoTimer) this["last_time_played"].toString()
            .toLong() else Calendar.getInstance().timeInMillis
    )
}

//MostPlayed information with song/audio information
fun MutableMap<String, Any>.toMostPlayedEntity(autoTimer: Boolean = false): OnMostPlayedEntity {
    return OnMostPlayedEntity(
        dbMPId = defaultId,
        this.toAudioEntity(),
        timePlayed = this["time_played"].toString().toLong(),
        lastTimePlayed = if (!autoTimer) this["last_time_played"].toString()
            .toLong() else Calendar.getInstance().timeInMillis,
        count = this["count"].toString().toInt()
    )
}

//Playlist information
fun MutableMap<String, Any>.toPlaylistEntity(autoTimer: Boolean = false): OnPlaylistEntity {
    return OnPlaylistEntity(
        playlistId = defaultId,
        playlistName = this["playlist_name"].toString(),
        playlistDataAdded = if (!autoTimer) this["playlist_date_added"].toString()
            .toLong() else Calendar.getInstance().timeInMillis,
        playlistDateModified = if (!autoTimer) this["playlist_date_modified"].toString()
            .toLong() else Calendar.getInstance().timeInMillis
    )
}

//Playlist_Songs information with playlist information and song/audio information
fun MutableMap<String, Any>.toPlaylistSongsEntity(autoTimer: Boolean = false): OnPlaylistSongsEntity {
    return OnPlaylistSongsEntity(
        dbPLSongsId = defaultId,
        OnPlaylistEntity(
            playlistId = this["playlist_id"].toString().toInt(),
            playlistName = this["playlist_name"].toString(),
            playlistDataAdded = if (!autoTimer) this["playlist_date_added"].toString()
                .toLong() else Calendar.getInstance().timeInMillis,
            playlistDateModified = if (!autoTimer) this["playlist_date_modified"].toString()
                .toLong() else Calendar.getInstance().timeInMillis
        ),
        this.toAudioEntity()
    )
}

//This part it's a Kotlin extension, here we will convert Entities from Kotlin and convert in Map/List.

//Basic song/audio information
fun OnSongEntity.toSongMutableMap(): MutableMap<String, Any> {
    val defaultMutableMap: MutableMap<String, Any> = HashMap()
    defaultMutableMap["last_data"] = this.lastData
    defaultMutableMap["display_name"] = this.displayName
    defaultMutableMap["id"] = this.id
    defaultMutableMap["album"] = this.album
    defaultMutableMap["album_id"] = this.albumId
    defaultMutableMap["artist"] = this.artist
    defaultMutableMap["artist_id"] = this.artistId
    defaultMutableMap["date_added"] = this.dateAdded
    defaultMutableMap["duration"] = this.duration
    defaultMutableMap["title"] = this.title
    defaultMutableMap["artwork"] = this.artwork
    return defaultMutableMap
}

//Favorites information with song/audio information
fun OnFavoritesEntity.toFavMutableMap(): MutableMap<String, Any> {
    return this.songEntity.toSongMutableMap()
}

//List of favorites information with song/audio information
fun List<OnFavoritesEntity>.toFavListAndMutableMap(reverse: Boolean = false): ArrayList<MutableMap<String, Any>> {
    val defaultListMutableMap: ArrayList<MutableMap<String, Any>> = ArrayList()
    this.forEach {
        defaultListMutableMap.add(it.toFavMutableMap())
    }
    return if (!reverse) defaultListMutableMap else ArrayList(defaultListMutableMap.reversed())
}

//LastPlayed information with song/audio information
fun OnLastPlayedEntity.toLastPlayedMutableMap(): MutableMap<String, Any> {
    val defaultMutableMap: MutableMap<String, Any> = HashMap()
    defaultMutableMap.putAll(this.songEntity.toSongMutableMap())
    defaultMutableMap["time_played"] = this.timePlayed
    defaultMutableMap["last_time_played"] = this.lastTimePlayed
    return defaultMutableMap
}

//List of last played information with song/audio information
fun List<OnLastPlayedEntity>.toLPListAndMutableMap(reverse: Boolean = false): ArrayList<MutableMap<String, Any>> {
    val defaultListMutableMap: ArrayList<MutableMap<String, Any>> = ArrayList()
    this.forEach {
        defaultListMutableMap.add(it.toLastPlayedMutableMap())
    }
    return if (!reverse) defaultListMutableMap else ArrayList(defaultListMutableMap.reversed())
}

//MostPlayed information with song/audio information
fun OnMostPlayedEntity.toMostPlayedMutableMap(): MutableMap<String, Any> {
    val defaultMutableMap: MutableMap<String, Any> = HashMap()
    defaultMutableMap.putAll(this.songEntity.toSongMutableMap())
    defaultMutableMap["time_played"] = this.timePlayed
    defaultMutableMap["last_time_played"] = this.lastTimePlayed
    defaultMutableMap["count"] = this.count
    return defaultMutableMap
}

//List of most played information with song/audio information
fun List<OnMostPlayedEntity>.toMPListAndMutableMap(reverse: Boolean = false): ArrayList<MutableMap<String, Any>> {
    val defaultListMutableMap: ArrayList<MutableMap<String, Any>> = ArrayList()
    this.forEach {
        defaultListMutableMap.add(it.toMostPlayedMutableMap())
    }
    return if (!reverse) defaultListMutableMap else ArrayList(defaultListMutableMap.reversed())
}

//Playlist information
fun OnPlaylistEntity.toPlaylistMutableMap(): MutableMap<String, Any> {
    val defaultMutableMap: MutableMap<String, Any> = HashMap()
    defaultMutableMap["playlist_id"] = this.playlistId
    defaultMutableMap["playlist_name"] = this.playlistName
    defaultMutableMap["playlist_date_added"] = this.playlistDataAdded
    defaultMutableMap["playlist_date_modified"] = this.playlistDateModified
    return defaultMutableMap
}

//List of playlist information
fun List<OnPlaylistEntity>.toPlaylistListAndMutableMap(reverse: Boolean = false): ArrayList<MutableMap<String, Any>> {
    val defaultListMutableMap: ArrayList<MutableMap<String, Any>> = ArrayList()
    this.forEach {
        defaultListMutableMap.add(it.toPlaylistMutableMap())
    }
    return if (!reverse) defaultListMutableMap else ArrayList(defaultListMutableMap.reversed())
}

//Playlist_Songs information with playlist information and song/audio information
fun OnPlaylistSongsEntity.toPlaylistSongsMutableMap(): MutableMap<String, Any> {
    val defaultMutableMap: MutableMap<String, Any> = HashMap()
    defaultMutableMap.putAll(this.playlistEntity.toPlaylistMutableMap())
    defaultMutableMap.putAll(this.songEntity.toSongMutableMap())
    return defaultMutableMap
}

//List of playlist songs information with playlist information and song/audio information
fun List<OnPlaylistSongsEntity>.toPlaylistSongsListAndMutableMap(reverse: Boolean = false): ArrayList<MutableMap<String, Any>> {
    val defaultListMutableMap: ArrayList<MutableMap<String, Any>> = ArrayList()
    this.forEach {
        defaultListMutableMap.add(it.toPlaylistSongsMutableMap())
    }
    return if (!reverse) defaultListMutableMap else ArrayList(defaultListMutableMap.reversed())
}

// Checking null values
fun Any?.isNull(): Boolean {
    return this == null
}
