package com.lucasjosino.on_audio_room.utils

import android.content.Context
import android.util.Log
import com.lucasjosino.on_audio_room.OnAudioRoomPlugin
import com.lucasjosino.on_audio_room.databases.OnAudioDatabase
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import java.io.File

//Database parameters
private const val onSharedPrefKeyDbName = "on_audio_db_name"
private const val onSharedPrefKeyDbLimit = "on_audio_db_limit"
const val channelError = "on_audio_error"

//Set the database name
//
//Important:
//
//- This method has created because, "on_audio_room" it's a public plugin and to avoid any type of
//error/bug with others apps add your [own] database name. I think that is important, but, optional.
//- This method has to be called [before] any other Add/Delete/Query method.
//- If the database name has changed after call any of the Add/Delete/Query methods you will lose all
//information stored.
fun setDatabaseName(context: Context, result: MethodChannel.Result, call: MethodCall) {
    val dbName = call.argument<String>("databaseName")
    val sp = context.getSharedPreferences("on_audio_room", Context.MODE_PRIVATE)
    if (dbName != null && !sp.contains(onSharedPrefKeyDbName)) {
        sp.edit().putString(onSharedPrefKeyDbName, dbName).apply()
        result.success(true)
    } else result.error(
        channelError,
        "[setDatabaseName]: databaseName is null or has already been defined.",
        "Set a non null value to databaseName or delete current name using [resetDatabaseName].")
}

//Set the database limit
//
//Why?:
//
//- Sometimes you want to limit the query to avoid too work/lagging.
//
//Important:
//
//- Default limit it's [50].
//- This limit it's used to limit the query(Like: queryAllFrom/queryAllPlaylists...), but, won't
//avoid [infinite] items inside database.
fun setDatabaseLimit(context: Context, result: MethodChannel.Result, call: MethodCall) {
    val dbLimit = call.argument<Int>("databaseLimit")
    val sp = context.getSharedPreferences("on_audio_room", Context.MODE_PRIVATE)
    if (dbLimit != null && !sp.contains(onSharedPrefKeyDbLimit)) {
        sp.edit().putInt(onSharedPrefKeyDbLimit, dbLimit).apply()
        result.success(true)
    } else result.error(
        channelError,
        "[setDatabaseLimit]: databaseLimit is null or has already been defined.",
        "Set a non null value to databaseLimit or delete current name using [resetDatabaseLimit].")
}

//Reset all the database.
//
//Important:
//
//- You will lose all database information and the old database will keep existing and consuming memory.
// TODO(Find a way to delete database file)
fun resetDatabaseName(context: Context, result: MethodChannel.Result) {
    val sp = context.getSharedPreferences("on_audio_room", Context.MODE_PRIVATE)
    if (sp.contains(onSharedPrefKeyDbName)) {
        sp.edit().remove(onSharedPrefKeyDbName).apply()
        result.success(true)
    } else result.error(
        channelError,
        "[resetDatabaseName]: You can only reset the databaseName if has already been defined.",
        "Use [setDatabaseName] to define a databaseName.")
}

//Reset the database limit
//
//Important:
//
//- The limit will be set to default[50].
fun resetDatabaseLimit(context: Context, result: MethodChannel.Result) {
    val sp = context.getSharedPreferences("on_audio_room", Context.MODE_PRIVATE)
    if (sp.contains(onSharedPrefKeyDbLimit)) {
        sp.edit().remove(onSharedPrefKeyDbLimit).apply()
        result.success(true)
    } else result.error(
        channelError,
        "[resetDatabaseLimit]: You can only reset the databaseLimit if has already been defined.",
        "Use [setDatabaseLimit] to define a databaseLimit.")
}

// Didn't work

//fun deleteDatabaseFile(context: Context) : Boolean {
//    return try {
//        if (!OnAudioDatabase.initDatabase(context).isOpen) {
//            val dbFiles = File(context.applicationInfo.dataDir + "/databases")
//            val db = File(dbFiles, OnAudioRoomPlugin().getDbName(context)!!)
//            db.delete()
//        } else false
//    } catch (e: Exception) { Log.i(channelError, e.toString()) ; false}
//}
